"""
CFspider IP 地图可视化模块

生成包含代理 IP 地理位置的 HTML 地图文件，使用 MapLibre GL JS。
"""
import json
from typing import Optional, List, Dict, Any
from datetime import datetime


# Cloudflare 节点代码对应的坐标（主要节点）
COLO_COORDINATES = {
    # 亚洲
    "NRT": {"lat": 35.6762, "lng": 139.6503, "city": "东京", "country": "日本"},
    "HND": {"lat": 35.5494, "lng": 139.7798, "city": "东京羽田", "country": "日本"},
    "KIX": {"lat": 34.4347, "lng": 135.2441, "city": "大阪", "country": "日本"},
    "HKG": {"lat": 22.3080, "lng": 113.9185, "city": "香港", "country": "香港"},
    "SIN": {"lat": 1.3521, "lng": 103.8198, "city": "新加坡", "country": "新加坡"},
    "ICN": {"lat": 37.4602, "lng": 126.4407, "city": "首尔", "country": "韩国"},
    "TPE": {"lat": 25.0777, "lng": 121.2330, "city": "台北", "country": "台湾"},
    "BKK": {"lat": 13.6900, "lng": 100.7501, "city": "曼谷", "country": "泰国"},
    "KUL": {"lat": 2.7456, "lng": 101.7072, "city": "吉隆坡", "country": "马来西亚"},
    "BOM": {"lat": 19.0896, "lng": 72.8656, "city": "孟买", "country": "印度"},
    "DEL": {"lat": 28.5562, "lng": 77.1000, "city": "新德里", "country": "印度"},
    "SYD": {"lat": -33.9399, "lng": 151.1753, "city": "悉尼", "country": "澳大利亚"},
    "MEL": {"lat": -37.6690, "lng": 144.8410, "city": "墨尔本", "country": "澳大利亚"},
    
    # 北美
    "SJC": {"lat": 37.3639, "lng": -121.9289, "city": "圣何塞", "country": "美国"},
    "LAX": {"lat": 33.9416, "lng": -118.4085, "city": "洛杉矶", "country": "美国"},
    "SEA": {"lat": 47.4502, "lng": -122.3088, "city": "西雅图", "country": "美国"},
    "DFW": {"lat": 32.8998, "lng": -97.0403, "city": "达拉斯", "country": "美国"},
    "ORD": {"lat": 41.9742, "lng": -87.9073, "city": "芝加哥", "country": "美国"},
    "IAD": {"lat": 38.9531, "lng": -77.4565, "city": "华盛顿", "country": "美国"},
    "EWR": {"lat": 40.6895, "lng": -74.1745, "city": "纽瓦克", "country": "美国"},
    "MIA": {"lat": 25.7959, "lng": -80.2870, "city": "迈阿密", "country": "美国"},
    "ATL": {"lat": 33.6407, "lng": -84.4277, "city": "亚特兰大", "country": "美国"},
    "YYZ": {"lat": 43.6777, "lng": -79.6248, "city": "多伦多", "country": "加拿大"},
    "YVR": {"lat": 49.1947, "lng": -123.1789, "city": "温哥华", "country": "加拿大"},
    
    # 欧洲
    "LHR": {"lat": 51.4700, "lng": -0.4543, "city": "伦敦", "country": "英国"},
    "CDG": {"lat": 49.0097, "lng": 2.5479, "city": "巴黎", "country": "法国"},
    "FRA": {"lat": 50.0379, "lng": 8.5622, "city": "法兰克福", "country": "德国"},
    "AMS": {"lat": 52.3105, "lng": 4.7683, "city": "阿姆斯特丹", "country": "荷兰"},
    "ZRH": {"lat": 47.4647, "lng": 8.5492, "city": "苏黎世", "country": "瑞士"},
    "MAD": {"lat": 40.4983, "lng": -3.5676, "city": "马德里", "country": "西班牙"},
    "MXP": {"lat": 45.6306, "lng": 8.7281, "city": "米兰", "country": "意大利"},
    "WAW": {"lat": 52.1672, "lng": 20.9679, "city": "华沙", "country": "波兰"},
    "ARN": {"lat": 59.6498, "lng": 17.9238, "city": "斯德哥尔摩", "country": "瑞典"},
    
    # 南美
    "GRU": {"lat": -23.4356, "lng": -46.4731, "city": "圣保罗", "country": "巴西"},
    "EZE": {"lat": -34.8222, "lng": -58.5358, "city": "布宜诺斯艾利斯", "country": "阿根廷"},
    "SCL": {"lat": -33.3930, "lng": -70.7858, "city": "圣地亚哥", "country": "智利"},
    
    # 中东/非洲
    "DXB": {"lat": 25.2532, "lng": 55.3657, "city": "迪拜", "country": "阿联酋"},
    "JNB": {"lat": -26.1392, "lng": 28.2460, "city": "约翰内斯堡", "country": "南非"},
    "CAI": {"lat": 30.1219, "lng": 31.4056, "city": "开罗", "country": "埃及"},
}


class IPMapCollector:
    """
    IP 地图数据收集器
    
    收集爬取过程中使用的代理 IP 信息，用于生成可视化地图。
    """
    
    def __init__(self):
        self.ip_records: List[Dict[str, Any]] = []
    
    def add_record(
        self,
        url: str,
        ip: Optional[str] = None,
        cf_colo: Optional[str] = None,
        cf_ray: Optional[str] = None,
        status_code: Optional[int] = None,
        response_time: Optional[float] = None
    ):
        """添加一条 IP 记录"""
        record = {
            "url": url,
            "ip": ip,
            "cf_colo": cf_colo,
            "cf_ray": cf_ray,
            "status_code": status_code,
            "response_time": response_time,
            "timestamp": datetime.now().isoformat()
        }
        
        # 获取节点坐标
        if cf_colo and cf_colo in COLO_COORDINATES:
            coord = COLO_COORDINATES[cf_colo]
            record["lat"] = coord["lat"]
            record["lng"] = coord["lng"]
            record["city"] = coord["city"]
            record["country"] = coord["country"]
        
        self.ip_records.append(record)
    
    def get_records(self) -> List[Dict[str, Any]]:
        """获取所有记录"""
        return self.ip_records
    
    def clear(self):
        """清空记录"""
        self.ip_records = []
    
    def get_unique_colos(self) -> List[str]:
        """获取唯一的节点代码列表"""
        colos = set()
        for record in self.ip_records:
            if record.get("cf_colo"):
                colos.add(record["cf_colo"])
        return list(colos)


# 全局收集器实例
_global_collector = IPMapCollector()


def get_collector() -> IPMapCollector:
    """获取全局 IP 收集器"""
    return _global_collector


def add_ip_record(
    url: str,
    ip: Optional[str] = None,
    cf_colo: Optional[str] = None,
    cf_ray: Optional[str] = None,
    status_code: Optional[int] = None,
    response_time: Optional[float] = None
):
    """添加 IP 记录到全局收集器"""
    _global_collector.add_record(
        url=url,
        ip=ip,
        cf_colo=cf_colo,
        cf_ray=cf_ray,
        status_code=status_code,
        response_time=response_time
    )


def generate_map_html(
    output_file: str = "cfspider_map.html",
    title: str = "CFspider Proxy IP Map",
    collector: Optional[IPMapCollector] = None
) -> str:
    """
    生成 IP 地图 HTML 文件
    
    Args:
        output_file: 输出文件名
        title: 页面标题
        collector: IP 收集器（默认使用全局收集器）
    
    Returns:
        生成的文件路径
    """
    if collector is None:
        collector = _global_collector
    
    records = collector.get_records()
    
    # 过滤有坐标的记录
    geo_records = [r for r in records if r.get("lat") and r.get("lng")]
    
    # 生成 GeoJSON 数据
    features = []
    for i, record in enumerate(geo_records):
        feature = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [record["lng"], record["lat"]]
            },
            "properties": {
                "id": i,
                "url": record.get("url", ""),
                "ip": record.get("ip", "N/A"),
                "cf_colo": record.get("cf_colo", "N/A"),
                "cf_ray": record.get("cf_ray", "N/A"),
                "city": record.get("city", "Unknown"),
                "country": record.get("country", "Unknown"),
                "status_code": record.get("status_code", 0),
                "response_time": record.get("response_time", 0),
                "timestamp": record.get("timestamp", "")
            }
        }
        features.append(feature)
    
    geojson = {
        "type": "FeatureCollection",
        "features": features
    }
    
    # 统计信息
    total_requests = len(records)
    geo_requests = len(geo_records)
    unique_colos = collector.get_unique_colos()
    
    html_content = f'''<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{title}</title>
    <script src="https://unpkg.com/maplibre-gl@3.6.2/dist/maplibre-gl.js"></script>
    <link href="https://unpkg.com/maplibre-gl@3.6.2/dist/maplibre-gl.css" rel="stylesheet" />
    <style>
        :root {{
            --bg-primary: #0a0a0f;
            --bg-secondary: #12121a;
            --neon-cyan: #00f5ff;
            --neon-magenta: #ff2d95;
            --neon-yellow: #f7f71c;
            --neon-green: #50fa7b;
            --text-primary: #ffffff;
            --text-secondary: #888888;
        }}
        
        * {{ margin: 0; padding: 0; box-sizing: border-box; }}
        
        body {{
            font-family: 'Segoe UI', 'Microsoft YaHei', sans-serif;
            background: var(--bg-primary);
            color: var(--text-primary);
        }}
        
        #map {{
            position: absolute;
            top: 0;
            bottom: 0;
            width: 100%;
        }}
        
        .info-panel {{
            position: absolute;
            top: 20px;
            left: 20px;
            background: rgba(10, 10, 15, 0.95);
            border: 1px solid var(--neon-cyan);
            border-radius: 12px;
            padding: 20px;
            min-width: 280px;
            box-shadow: 0 0 30px rgba(0, 245, 255, 0.3);
            z-index: 1000;
        }}
        
        .info-panel h1 {{
            font-size: 1.4rem;
            color: var(--neon-cyan);
            margin-bottom: 15px;
            text-shadow: 0 0 10px var(--neon-cyan);
        }}
        
        .stats {{
            display: grid;
            grid-template-columns: repeat(2, 1fr);
            gap: 10px;
            margin-bottom: 15px;
        }}
        
        .stat-item {{
            background: rgba(0, 245, 255, 0.1);
            border: 1px solid rgba(0, 245, 255, 0.3);
            border-radius: 8px;
            padding: 10px;
            text-align: center;
        }}
        
        .stat-value {{
            font-size: 1.5rem;
            font-weight: bold;
            color: var(--neon-cyan);
        }}
        
        .stat-label {{
            font-size: 0.75rem;
            color: var(--text-secondary);
            margin-top: 5px;
        }}
        
        .colo-list {{
            background: rgba(255, 255, 255, 0.05);
            border-radius: 8px;
            padding: 10px;
            max-height: 150px;
            overflow-y: auto;
        }}
        
        .colo-tag {{
            display: inline-block;
            background: rgba(255, 45, 149, 0.2);
            border: 1px solid var(--neon-magenta);
            color: var(--neon-magenta);
            padding: 4px 8px;
            border-radius: 4px;
            font-size: 0.75rem;
            margin: 2px;
        }}
        
        .maplibregl-popup-content {{
            background: rgba(10, 10, 15, 0.95) !important;
            border: 1px solid var(--neon-cyan) !important;
            border-radius: 8px !important;
            padding: 15px !important;
            color: var(--text-primary) !important;
            box-shadow: 0 0 20px rgba(0, 245, 255, 0.3) !important;
        }}
        
        .maplibregl-popup-close-button {{
            color: var(--neon-cyan) !important;
            font-size: 20px !important;
        }}
        
        .popup-title {{
            color: var(--neon-cyan);
            font-size: 1.1rem;
            font-weight: bold;
            margin-bottom: 10px;
        }}
        
        .popup-row {{
            display: flex;
            justify-content: space-between;
            padding: 5px 0;
            border-bottom: 1px solid rgba(255, 255, 255, 0.1);
        }}
        
        .popup-label {{
            color: var(--text-secondary);
        }}
        
        .popup-value {{
            color: var(--neon-green);
            font-family: monospace;
        }}
        
        .footer {{
            position: absolute;
            bottom: 20px;
            left: 20px;
            background: rgba(10, 10, 15, 0.9);
            border: 1px solid rgba(255, 255, 255, 0.2);
            border-radius: 8px;
            padding: 10px 15px;
            font-size: 0.8rem;
            color: var(--text-secondary);
            z-index: 1000;
        }}
        
        .footer a {{
            color: var(--neon-cyan);
            text-decoration: none;
        }}
    </style>
</head>
<body>
    <div id="map"></div>
    
    <div class="info-panel">
        <h1>CFspider IP Map</h1>
        <div class="stats">
            <div class="stat-item">
                <div class="stat-value">{total_requests}</div>
                <div class="stat-label">Total Requests</div>
            </div>
            <div class="stat-item">
                <div class="stat-value">{len(unique_colos)}</div>
                <div class="stat-label">Unique Nodes</div>
            </div>
        </div>
        <div class="colo-list">
            {"".join([f'<span class="colo-tag">{colo}</span>' for colo in unique_colos]) if unique_colos else '<span style="color: #888;">No data</span>'}
        </div>
    </div>
    
    <div class="footer">
        Generated by <a href="https://github.com/violettoolssite/CFspider" target="_blank">CFspider</a> | 
        {datetime.now().strftime("%Y-%m-%d %H:%M:%S")}
    </div>
    
    <script>
        const geojsonData = {json.dumps(geojson)};
        
        const map = new maplibregl.Map({{
            container: 'map',
            style: {{
                version: 8,
                sources: {{
                    'carto-dark': {{
                        type: 'raster',
                        tiles: [
                            'https://a.basemaps.cartocdn.com/dark_all/{{z}}/{{x}}/{{y}}@2x.png',
                            'https://b.basemaps.cartocdn.com/dark_all/{{z}}/{{x}}/{{y}}@2x.png',
                            'https://c.basemaps.cartocdn.com/dark_all/{{z}}/{{x}}/{{y}}@2x.png'
                        ],
                        tileSize: 256
                    }}
                }},
                layers: [{{
                    id: 'carto-dark-layer',
                    type: 'raster',
                    source: 'carto-dark',
                    minzoom: 0,
                    maxzoom: 19
                }}]
            }},
            center: [0, 20],
            zoom: 1.5
        }});
        
        map.addControl(new maplibregl.NavigationControl());
        
        map.on('load', function() {{
            // 添加数据源
            map.addSource('proxies', {{
                type: 'geojson',
                data: geojsonData
            }});
            
            // 添加圆点图层
            map.addLayer({{
                id: 'proxy-points',
                type: 'circle',
                source: 'proxies',
                paint: {{
                    'circle-radius': 10,
                    'circle-color': '#00f5ff',
                    'circle-opacity': 0.8,
                    'circle-stroke-width': 2,
                    'circle-stroke-color': '#ff2d95'
                }}
            }});
            
            // 添加发光效果
            map.addLayer({{
                id: 'proxy-points-glow',
                type: 'circle',
                source: 'proxies',
                paint: {{
                    'circle-radius': 20,
                    'circle-color': '#00f5ff',
                    'circle-opacity': 0.2,
                    'circle-blur': 1
                }}
            }}, 'proxy-points');
            
            // 点击事件
            map.on('click', 'proxy-points', function(e) {{
                const props = e.features[0].properties;
                const coordinates = e.features[0].geometry.coordinates.slice();
                
                const popupContent = `
                    <div class="popup-title">${{props.city}}, ${{props.country}}</div>
                    <div class="popup-row">
                        <span class="popup-label">Node:</span>
                        <span class="popup-value">${{props.cf_colo}}</span>
                    </div>
                    <div class="popup-row">
                        <span class="popup-label">IP:</span>
                        <span class="popup-value">${{props.ip}}</span>
                    </div>
                    <div class="popup-row">
                        <span class="popup-label">Status:</span>
                        <span class="popup-value">${{props.status_code}}</span>
                    </div>
                    <div class="popup-row">
                        <span class="popup-label">Time:</span>
                        <span class="popup-value">${{props.response_time ? props.response_time.toFixed(2) + 'ms' : 'N/A'}}</span>
                    </div>
                    <div class="popup-row" style="border: none;">
                        <span class="popup-label">URL:</span>
                        <span class="popup-value" style="font-size: 0.7rem; word-break: break-all;">${{props.url.substring(0, 40)}}...</span>
                    </div>
                `;
                
                new maplibregl.Popup()
                    .setLngLat(coordinates)
                    .setHTML(popupContent)
                    .addTo(map);
            }});
            
            // 鼠标样式
            map.on('mouseenter', 'proxy-points', function() {{
                map.getCanvas().style.cursor = 'pointer';
            }});
            
            map.on('mouseleave', 'proxy-points', function() {{
                map.getCanvas().style.cursor = '';
            }});
            
            // 如果有数据，自动缩放到数据范围
            if (geojsonData.features.length > 0) {{
                const bounds = new maplibregl.LngLatBounds();
                geojsonData.features.forEach(feature => {{
                    bounds.extend(feature.geometry.coordinates);
                }});
                map.fitBounds(bounds, {{ padding: 50 }});
            }}
        }});
    </script>
</body>
</html>'''
    
    # 写入文件
    with open(output_file, 'w', encoding='utf-8') as f:
        f.write(html_content)
    
    return output_file


def clear_records():
    """清空全局收集器的记录"""
    _global_collector.clear()




"""
CFspider Session 模块

提供会话管理功能，在多个请求之间保持代理配置、请求头和 Cookie。
"""

from .api import request


class Session:
    """
    CFspider 会话类
    
    在多个请求之间保持相同的代理配置、请求头和 Cookie。
    适合需要登录状态或连续请求的场景。
    
    Attributes:
        cf_proxies (str): Workers 代理地址
        headers (dict): 会话级别的默认请求头
        cookies (dict): 会话级别的 Cookie
    
    Example:
        >>> import cfspider
        >>> 
        >>> # 创建会话
        >>> with cfspider.Session(cf_proxies="https://your-workers.dev") as session:
        ...     # 设置会话级别的请求头
        ...     session.headers['Authorization'] = 'Bearer token'
        ...     
        ...     # 所有请求都会使用相同的代理和请求头
        ...     response1 = session.get("https://api.example.com/user")
        ...     response2 = session.post("https://api.example.com/data", json={"key": "value"})
        ...     
        ...     # Cookie 会自动保持
        ...     print(session.cookies)
    
    Note:
        如果需要隐身模式的会话一致性（自动 Referer、随机延迟等），
        请使用 cfspider.StealthSession。
    """
    
    def __init__(self, cf_proxies=None):
        """
        初始化会话
        
        Args:
            cf_proxies (str): Workers 代理地址（必填）
                例如："https://your-workers.dev"
        
        Raises:
            ValueError: 当 cf_proxies 为空时
        
        Example:
            >>> session = cfspider.Session(cf_proxies="https://your-workers.dev")
        """
        if not cf_proxies:
            raise ValueError(
                "cf_proxies 是必填参数。\n"
                "请提供 CFspider Workers 地址，例如：\n"
                "  session = cfspider.Session(cf_proxies='https://your-workers.dev')\n\n"
                "如果不需要代理，可以直接使用 cfspider.get() 等函数。\n"
                "如果需要隐身模式会话，请使用 cfspider.StealthSession。"
            )
        self.cf_proxies = cf_proxies.rstrip("/")
        self.headers = {}
        self.cookies = {}
    
    def request(self, method, url, **kwargs):
        """
        发送 HTTP 请求
        
        Args:
            method (str): HTTP 方法（GET, POST, PUT, DELETE 等）
            url (str): 目标 URL
            **kwargs: 其他参数，与 cfspider.request() 相同
        
        Returns:
            CFSpiderResponse: 响应对象
        
        Note:
            会话级别的 headers 和 cookies 会自动添加到请求中，
            但请求级别的参数优先级更高。
        """
        headers = self.headers.copy()
        headers.update(kwargs.pop("headers", {}))
        
        cookies = self.cookies.copy()
        cookies.update(kwargs.pop("cookies", {}))
        
        return request(
            method,
            url,
            cf_proxies=self.cf_proxies,
            headers=headers,
            cookies=cookies,
            **kwargs
        )
    
    def get(self, url, **kwargs):
        """发送 GET 请求"""
        return self.request("GET", url, **kwargs)
    
    def post(self, url, **kwargs):
        """发送 POST 请求"""
        return self.request("POST", url, **kwargs)
    
    def put(self, url, **kwargs):
        """发送 PUT 请求"""
        return self.request("PUT", url, **kwargs)
    
    def delete(self, url, **kwargs):
        """发送 DELETE 请求"""
        return self.request("DELETE", url, **kwargs)
    
    def head(self, url, **kwargs):
        """发送 HEAD 请求"""
        return self.request("HEAD", url, **kwargs)
    
    def options(self, url, **kwargs):
        """发送 OPTIONS 请求"""
        return self.request("OPTIONS", url, **kwargs)
    
    def patch(self, url, **kwargs):
        """发送 PATCH 请求"""
        return self.request("PATCH", url, **kwargs)
    
    def close(self):
        """
        关闭会话
        
        当前实现中，每个请求都是独立的，无需特殊清理。
        保留此方法是为了与 requests.Session 保持接口兼容。
        """
        pass
    
    def __enter__(self):
        """支持上下文管理器（with 语句）"""
        return self
    
    def __exit__(self, *args):
        """退出上下文时关闭会话"""
        self.close()




"""
CFspider 内置 VLESS 客户端
通过 WebSocket 连接 edgetunnel，提供本地 HTTP 代理
"""

import socket
import struct
import threading
import ssl
import time
import uuid
from urllib.parse import urlparse


class VlessClient:
    """VLESS 协议客户端"""
    
    def __init__(self, ws_url, vless_uuid=None):
        """
        初始化 VLESS 客户端
        
        Args:
            ws_url: edgetunnel WebSocket 地址，如 "wss://v2.kami666.xyz"
            vless_uuid: VLESS UUID，如不提供则自动生成
        """
        self.ws_url = ws_url
        self.vless_uuid = vless_uuid or str(uuid.uuid4())
        
        parsed = urlparse(ws_url)
        self.host = parsed.hostname
        self.port = parsed.port or (443 if parsed.scheme == 'wss' else 80)
        self.path = parsed.path or '/'
        self.use_ssl = parsed.scheme == 'wss'
    
    def _create_vless_header(self, target_host, target_port):
        """创建 VLESS 请求头"""
        # VLESS 协议版本
        header = bytes([0])
        
        # UUID (16 bytes)
        uuid_bytes = uuid.UUID(self.vless_uuid).bytes
        header += uuid_bytes
        
        # 附加信息长度
        header += bytes([0])
        
        # 命令 (1 = TCP)
        header += bytes([1])
        
        # 目标端口
        header += struct.pack('>H', target_port)
        
        # 地址类型和地址
        try:
            # 尝试解析为 IPv4
            socket.inet_aton(target_host)
            header += bytes([1])  # IPv4
            header += socket.inet_aton(target_host)
        except socket.error:
            try:
                # 尝试解析为 IPv6
                socket.inet_pton(socket.AF_INET6, target_host)
                header += bytes([3])  # IPv6
                header += socket.inet_pton(socket.AF_INET6, target_host)
            except socket.error:
                # 域名
                header += bytes([2])  # 域名
                domain_bytes = target_host.encode('utf-8')
                header += bytes([len(domain_bytes)])
                header += domain_bytes
        
        return header
    
    def _websocket_handshake(self, sock):
        """执行 WebSocket 握手"""
        import base64
        import hashlib
        import os
        
        # 生成随机 key
        key = base64.b64encode(os.urandom(16)).decode('utf-8')
        
        # 构建握手请求
        request = (
            f"GET {self.path} HTTP/1.1\r\n"
            f"Host: {self.host}\r\n"
            f"Upgrade: websocket\r\n"
            f"Connection: Upgrade\r\n"
            f"Sec-WebSocket-Key: {key}\r\n"
            f"Sec-WebSocket-Version: 13\r\n"
            f"\r\n"
        )
        
        sock.sendall(request.encode('utf-8'))
        
        # 读取响应
        response = b''
        while b'\r\n\r\n' not in response:
            chunk = sock.recv(1024)
            if not chunk:
                raise Exception("WebSocket 握手失败")
            response += chunk
        
        if b'101' not in response:
            raise Exception(f"WebSocket 握手失败: {response.decode('utf-8', errors='ignore')}")
        
        return True
    
    def _send_ws_frame(self, sock, data):
        """发送 WebSocket 帧"""
        import os
        
        # 构建帧头
        frame = bytes([0x82])  # Binary frame, FIN=1
        
        length = len(data)
        if length <= 125:
            frame += bytes([0x80 | length])  # Masked
        elif length <= 65535:
            frame += bytes([0x80 | 126])
            frame += struct.pack('>H', length)
        else:
            frame += bytes([0x80 | 127])
            frame += struct.pack('>Q', length)
        
        # 掩码
        mask = os.urandom(4)
        frame += mask
        
        # 掩码数据
        masked_data = bytes([data[i] ^ mask[i % 4] for i in range(len(data))])
        frame += masked_data
        
        sock.sendall(frame)
    
    def _recv_ws_frame(self, sock):
        """接收 WebSocket 帧"""
        # 读取帧头
        header = sock.recv(2)
        if len(header) < 2:
            return None
        
        opcode = header[0] & 0x0F
        masked = (header[1] & 0x80) != 0
        length = header[1] & 0x7F
        
        if length == 126:
            length_bytes = sock.recv(2)
            length = struct.unpack('>H', length_bytes)[0]
        elif length == 127:
            length_bytes = sock.recv(8)
            length = struct.unpack('>Q', length_bytes)[0]
        
        if masked:
            mask = sock.recv(4)
        
        # 读取数据
        data = b''
        while len(data) < length:
            chunk = sock.recv(length - len(data))
            if not chunk:
                break
            data += chunk
        
        if masked:
            data = bytes([data[i] ^ mask[i % 4] for i in range(len(data))])
        
        # 处理关闭帧
        if opcode == 0x08:
            return None
        
        return data
    
    def connect(self, target_host, target_port):
        """
        通过 VLESS 连接到目标
        
        Returns:
            VlessConnection: 可用于读写的连接对象
        """
        # 创建连接
        sock = socket.create_connection((self.host, self.port), timeout=30)
        
        if self.use_ssl:
            context = ssl.create_default_context()
            sock = context.wrap_socket(sock, server_hostname=self.host)
        
        # WebSocket 握手
        self._websocket_handshake(sock)
        
        # 创建 VLESS 头（稍后与第一个数据包一起发送）
        vless_header = self._create_vless_header(target_host, target_port)
        
        return VlessConnection(sock, self, vless_header)


class VlessConnection:
    """VLESS 连接封装"""
    
    def __init__(self, sock, client, vless_header=None):
        self.sock = sock
        self.client = client
        self.buffer = b''
        self.first_response = True
        self.vless_header = vless_header  # 第一次发送时需要带上
        self.first_send = True
    
    def send(self, data):
        """发送数据"""
        if self.first_send and self.vless_header:
            # 第一次发送时，将 VLESS 头和数据一起发送
            self.client._send_ws_frame(self.sock, self.vless_header + data)
            self.first_send = False
        else:
            self.client._send_ws_frame(self.sock, data)
    
    def recv(self, size):
        """接收数据"""
        # 如果缓冲区不够，尝试接收更多数据
        if len(self.buffer) < size:
            try:
                frame = self.client._recv_ws_frame(self.sock)
                if frame:
                    # 第一个响应需要跳过 VLESS 响应头
                    if self.first_response and len(frame) >= 2:
                        addon_len = frame[1] if len(frame) > 1 else 0
                        frame = frame[2 + addon_len:]
                        self.first_response = False
                    self.buffer += frame
            except:
                pass
        
        result = self.buffer[:size]
        self.buffer = self.buffer[size:]
        return result
    
    def recv_all(self):
        """接收所有可用数据"""
        try:
            self.sock.setblocking(False)
            while True:
                try:
                    frame = self.client._recv_ws_frame(self.sock)
                    if frame is None:
                        break
                    
                    if self.first_response and len(frame) >= 2:
                        addon_len = frame[1] if len(frame) > 1 else 0
                        frame = frame[2 + addon_len:]
                        self.first_response = False
                    
                    self.buffer += frame
                except (BlockingIOError, ssl.SSLWantReadError):
                    break
        finally:
            self.sock.setblocking(True)
        
        result = self.buffer
        self.buffer = b''
        return result
    
    def close(self):
        """关闭连接"""
        try:
            self.sock.close()
        except:
            pass


class LocalVlessProxy:
    """本地 VLESS HTTP 代理服务器"""
    
    def __init__(self, ws_url, vless_uuid=None):
        """
        初始化本地代理
        
        Args:
            ws_url: edgetunnel WebSocket 地址
            vless_uuid: VLESS UUID
        """
        self.ws_url = ws_url
        self.vless_uuid = vless_uuid
        self.server = None
        self.thread = None
        self.port = None
        self.running = False
    
    def start(self):
        """启动代理服务器"""
        # 找可用端口
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server.bind(('127.0.0.1', 0))
        self.port = self.server.getsockname()[1]
        self.server.listen(10)
        
        self.running = True
        self.thread = threading.Thread(target=self._serve, daemon=True)
        self.thread.start()
        
        # 等待服务器就绪
        time.sleep(0.1)
        return self.port
    
    def _serve(self):
        """服务循环"""
        self.server.settimeout(1)
        while self.running:
            try:
                client, addr = self.server.accept()
                handler = threading.Thread(
                    target=self._handle_client,
                    args=(client,),
                    daemon=True
                )
                handler.start()
            except socket.timeout:
                continue
            except:
                break
    
    def _handle_client(self, client):
        """处理客户端连接"""
        try:
            client.settimeout(30)
            
            # 读取请求
            request = b''
            while b'\r\n\r\n' not in request:
                chunk = client.recv(4096)
                if not chunk:
                    return
                request += chunk
            
            # 解析请求
            lines = request.split(b'\r\n')
            first_line = lines[0].decode('utf-8')
            parts = first_line.split(' ')
            
            if len(parts) < 2:
                return
            
            method = parts[0]
            
            if method == 'CONNECT':
                # HTTPS 代理
                target = parts[1]
                if ':' in target:
                    host, port = target.rsplit(':', 1)
                    port = int(port)
                else:
                    host = target
                    port = 443
                
                self._handle_connect(client, host, port)
            else:
                # HTTP 代理
                url = parts[1]
                self._handle_http(client, method, url, request)
                
        except Exception as e:
            pass
        finally:
            try:
                client.close()
            except:
                pass
    
    def _handle_connect(self, client, host, port):
        """处理 HTTPS CONNECT 请求"""
        try:
            # 连接到 VLESS
            vless = VlessClient(self.ws_url, self.vless_uuid)
            conn = vless.connect(host, port)
            
            # 发送连接成功
            client.sendall(b'HTTP/1.1 200 Connection Established\r\n\r\n')
            
            # 双向转发（使用线程）
            self._relay_bidirectional(client, conn)
            
        except Exception as e:
            try:
                client.sendall(b'HTTP/1.1 502 Bad Gateway\r\n\r\n')
            except:
                pass
    
    def _handle_http(self, client, method, url, original_request):
        """处理 HTTP 请求"""
        try:
            parsed = urlparse(url)
            host = parsed.hostname
            port = parsed.port or 80
            path = parsed.path or '/'
            if parsed.query:
                path += '?' + parsed.query
            
            # 连接到 VLESS
            vless = VlessClient(self.ws_url, self.vless_uuid)
            conn = vless.connect(host, port)
            
            # 重建请求
            lines = original_request.split(b'\r\n')
            lines[0] = f'{method} {path} HTTP/1.1'.encode('utf-8')
            
            # 更新 Host 头
            new_lines = [lines[0]]
            has_host = False
            for line in lines[1:]:
                if line.lower().startswith(b'host:'):
                    new_lines.append(f'Host: {host}'.encode('utf-8'))
                    has_host = True
                elif line.lower().startswith(b'proxy-'):
                    continue
                else:
                    new_lines.append(line)
            
            if not has_host:
                new_lines.insert(1, f'Host: {host}'.encode('utf-8'))
            
            request = b'\r\n'.join(new_lines)
            conn.send(request)
            
            # 读取响应并转发
            self._relay_response(client, conn)
            
        except Exception as e:
            client.sendall(b'HTTP/1.1 502 Bad Gateway\r\n\r\n')
    
    def _relay_bidirectional(self, client, conn):
        """双向数据转发（使用线程）"""
        import threading
        
        stop_event = threading.Event()
        
        def client_to_vless():
            try:
                while not stop_event.is_set():
                    try:
                        client.settimeout(1)
                        data = client.recv(8192)
                        if data:
                            conn.send(data)
                        else:
                            break
                    except socket.timeout:
                        continue
                    except:
                        break
            finally:
                stop_event.set()
        
        def vless_to_client():
            try:
                while not stop_event.is_set():
                    try:
                        conn.sock.settimeout(1)
                        frame = self._recv_ws_frame_safe(conn)
                        if frame:
                            client.sendall(frame)
                        elif frame is None:
                            break
                    except socket.timeout:
                        continue
                    except:
                        break
            finally:
                stop_event.set()
        
        t1 = threading.Thread(target=client_to_vless, daemon=True)
        t2 = threading.Thread(target=vless_to_client, daemon=True)
        t1.start()
        t2.start()
        
        # 等待任一方向结束
        while not stop_event.is_set():
            time.sleep(0.1)
        
        conn.close()
    
    def _recv_ws_frame_safe(self, conn):
        """安全地接收 WebSocket 帧"""
        try:
            sock = conn.sock
            header = sock.recv(2)
            if len(header) < 2:
                return None
            
            opcode = header[0] & 0x0F
            masked = (header[1] & 0x80) != 0
            length = header[1] & 0x7F
            
            if length == 126:
                length_bytes = sock.recv(2)
                length = struct.unpack('>H', length_bytes)[0]
            elif length == 127:
                length_bytes = sock.recv(8)
                length = struct.unpack('>Q', length_bytes)[0]
            
            if masked:
                mask = sock.recv(4)
            
            data = b''
            while len(data) < length:
                chunk = sock.recv(min(length - len(data), 8192))
                if not chunk:
                    break
                data += chunk
            
            if masked:
                data = bytes([data[i] ^ mask[i % 4] for i in range(len(data))])
            
            if opcode == 0x08:
                return None
            
            # 跳过 VLESS 响应头
            if conn.first_response and len(data) >= 2:
                addon_len = data[1]
                data = data[2 + addon_len:]
                conn.first_response = False
            
            return data
        except:
            return b''
    
    def _relay_response(self, client, conn):
        """转发 HTTP 响应"""
        try:
            # 读取响应
            response = b''
            while True:
                data = conn.recv(8192)
                if not data:
                    break
                response += data
                client.sendall(data)
                
                # 检查是否完成
                if b'\r\n\r\n' in response:
                    # 简单处理：继续读取直到没有数据
                    conn.sock.settimeout(0.5)
                    try:
                        while True:
                            data = conn.recv(8192)
                            if not data:
                                break
                            client.sendall(data)
                    except socket.timeout:
                        pass
                    break
        finally:
            conn.close()
    
    def stop(self):
        """停止代理服务器"""
        self.running = False
        if self.server:
            try:
                self.server.close()
            except:
                pass
        self.server = None
        self.thread = None
        self.port = None
    
    @property
    def proxy_url(self):
        """获取代理 URL"""
        if self.port:
            return f"http://127.0.0.1:{self.port}"
        return None




"""
CFspider 核心 API 模块

提供同步 HTTP 请求功能，支持：
- 通过 Cloudflare Workers 代理请求
- TLS 指纹模拟 (curl_cffi)
- HTTP/2 支持 (httpx)
- 隐身模式（完整浏览器请求头）
- IP 地图可视化
"""

import requests
import time
from urllib.parse import urlencode, quote
from typing import Optional, Any

# 延迟导入 IP 地图模块
from . import ip_map

# 延迟导入 httpx，仅在需要 HTTP/2 时使用
_httpx = None

def _get_httpx():
    """延迟加载 httpx 模块"""
    global _httpx
    if _httpx is None:
        try:
            import httpx
            _httpx = httpx
        except ImportError:
            raise ImportError(
                "httpx is required for HTTP/2 support. "
                "Install it with: pip install httpx[http2]"
            )
    return _httpx


# 延迟导入 curl_cffi，仅在需要 TLS 指纹时使用
_curl_cffi = None

def _get_curl_cffi():
    """延迟加载 curl_cffi 模块"""
    global _curl_cffi
    if _curl_cffi is None:
        try:
            from curl_cffi import requests as curl_requests
            _curl_cffi = curl_requests
        except ImportError:
            raise ImportError(
                "curl_cffi is required for TLS fingerprint impersonation. "
                "Install it with: pip install curl_cffi"
            )
    return _curl_cffi


class CFSpiderResponse:
    """
    CFspider 响应对象
    
    封装 HTTP 响应，提供与 requests.Response 兼容的接口，
    并额外提供 Cloudflare 特有的信息（如节点代码、Ray ID）。
    
    Attributes:
        cf_colo (str): Cloudflare 数据中心代码（如 NRT=东京, SIN=新加坡, LAX=洛杉矶）
                       使用 Workers 代理时可用，表示请求经过的 CF 节点
        cf_ray (str): Cloudflare Ray ID，每个请求的唯一标识符
                      可用于调试和追踪请求
        text (str): 响应文本内容（自动解码）
        content (bytes): 响应原始字节内容
        status_code (int): HTTP 状态码（如 200, 404, 500）
        headers (dict): 响应头字典
        cookies: 响应 Cookie
        url (str): 最终请求的 URL（跟随重定向后）
        encoding (str): 响应编码
    
    Methods:
        json(**kwargs): 将响应解析为 JSON
        raise_for_status(): 当状态码非 2xx 时抛出 HTTPError
    
    Example:
        >>> response = cfspider.get("https://httpbin.org/ip", cf_proxies="...")
        >>> print(response.status_code)  # 200
        >>> print(response.cf_colo)      # NRT (东京节点)
        >>> print(response.cf_ray)       # 8a1b2c3d4e5f-NRT
        >>> data = response.json()
        >>> print(data['origin'])        # Cloudflare IP
    """
    
    def __init__(self, response, cf_colo=None, cf_ray=None):
        """
        初始化响应对象
        
        Args:
            response: 原始 requests/httpx/curl_cffi 响应对象
            cf_colo: Cloudflare 数据中心代码（从响应头获取）
            cf_ray: Cloudflare Ray ID（从响应头获取）
        """
        self._response = response
        self.cf_colo = cf_colo
        self.cf_ray = cf_ray
    
    @property
    def text(self) -> str:
        """响应文本内容（自动解码）"""
        return self._response.text
    
    @property
    def content(self) -> bytes:
        """响应原始字节内容"""
        return self._response.content
    
    @property
    def status_code(self) -> int:
        """HTTP 状态码"""
        return self._response.status_code
    
    @property
    def headers(self):
        """响应头字典"""
        return self._response.headers
    
    @property
    def cookies(self):
        """响应 Cookie"""
        return self._response.cookies
    
    @property
    def url(self) -> str:
        """最终请求的 URL（跟随重定向后）"""
        return self._response.url
    
    @property
    def encoding(self) -> Optional[str]:
        """响应编码"""
        return self._response.encoding
    
    @encoding.setter
    def encoding(self, value: str):
        """设置响应编码"""
        self._response.encoding = value
    
    def json(self, **kwargs) -> Any:
        """
        将响应解析为 JSON
        
        Args:
            **kwargs: 传递给 json.loads() 的参数
            
        Returns:
            解析后的 JSON 数据（dict 或 list）
            
        Raises:
            JSONDecodeError: 当响应不是有效的 JSON 时
        """
        return self._response.json(**kwargs)
    
    def raise_for_status(self):
        """
        当状态码非 2xx 时抛出 HTTPError
        
        Raises:
            requests.HTTPError: 当状态码表示错误时
        """
        self._response.raise_for_status()


def request(method, url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None, 
             map_output=False, map_file="cfspider_map.html", 
             stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """
    发送 HTTP 请求
    
    这是 CFspider 的核心函数，支持多种代理模式和反爬虫功能。
    
    Args:
        method (str): HTTP 方法（GET, POST, PUT, DELETE, HEAD, OPTIONS, PATCH）
        url (str): 目标 URL，必须包含协议（https://）
        cf_proxies (str, optional): 代理地址，根据 cf_workers 参数有不同含义：
            - 当 cf_workers=True 时：填写 CFspider Workers 地址（如 "https://your-workers.dev"）
            - 当 cf_workers=False 时：填写普通 HTTP/SOCKS5 代理（如 "http://127.0.0.1:8080"）
            - 不填写时：直接请求目标 URL，不使用代理
        cf_workers (bool): 是否使用 CFspider Workers API（默认 True）
            - True: cf_proxies 是 Workers 地址，请求通过 Workers API 转发
            - False: cf_proxies 是普通代理，使用 requests/httpx 的 proxies 参数
        http2 (bool): 是否启用 HTTP/2 协议（默认 False）
            - True: 使用 httpx 客户端，支持 HTTP/2
            - False: 使用 requests 库（默认行为）
            - 注意：http2 和 impersonate 不能同时使用
        impersonate (str, optional): TLS 指纹模拟，模拟真实浏览器的 TLS 握手特征
            - 可选值：chrome131, chrome124, safari18_0, firefox133, edge101 等
            - 设置后自动使用 curl_cffi 发送请求
            - 完整列表：cfspider.get_supported_browsers()
        map_output (bool): 是否生成 IP 地图 HTML 文件（默认 False）
            - True: 请求完成后生成包含代理 IP 信息的交互式地图
        map_file (str): 地图输出文件名（默认 "cfspider_map.html"）
        stealth (bool): 是否启用隐身模式（默认 False）
            - True: 自动添加 15+ 个完整浏览器请求头，模拟真实浏览器访问
            - 添加的请求头包括：User-Agent, Accept, Accept-Language, Sec-Fetch-*, Sec-CH-UA 等
        stealth_browser (str): 隐身模式使用的浏览器类型（默认 'chrome'）
            - 可选值：chrome, firefox, safari, edge, chrome_mobile
        delay (tuple, optional): 请求前的随机延迟范围（秒）
            - 如 (1, 3) 表示请求前随机等待 1-3 秒
            - 用于模拟人类行为，避免被反爬系统检测
        **kwargs: 其他参数，与 requests 库完全兼容
            - params (dict): URL 查询参数
            - headers (dict): 自定义请求头（会与隐身模式头合并）
            - data (dict/str): 表单数据
            - json (dict): JSON 数据（自动设置 Content-Type）
            - cookies (dict): Cookie
            - timeout (int/float): 超时时间（秒），默认 30
            - allow_redirects (bool): 是否跟随重定向，默认 True
            - verify (bool): 是否验证 SSL 证书，默认 True
    
    Returns:
        CFSpiderResponse: 响应对象，包含以下属性：
            - text: 响应文本
            - content: 响应字节
            - json(): 解析 JSON
            - status_code: HTTP 状态码
            - headers: 响应头
            - cf_colo: Cloudflare 节点代码（使用 Workers 时可用）
            - cf_ray: Cloudflare Ray ID
    
    Raises:
        ImportError: 当需要的可选依赖未安装时
            - http2=True 需要 httpx[http2]
            - impersonate 需要 curl_cffi
        ValueError: 当 http2 和 impersonate 同时启用时
        requests.RequestException: 网络请求失败时
    
    Examples:
        >>> import cfspider
        >>> 
        >>> # 基本 GET 请求
        >>> response = cfspider.get("https://httpbin.org/ip")
        >>> print(response.json())
        >>> 
        >>> # 使用 Workers 代理
        >>> response = cfspider.get(
        ...     "https://httpbin.org/ip",
        ...     cf_proxies="https://your-workers.dev"
        ... )
        >>> print(response.cf_colo)  # NRT, SIN, LAX 等
        >>> 
        >>> # 隐身模式 + TLS 指纹
        >>> response = cfspider.get(
        ...     "https://example.com",
        ...     stealth=True,
        ...     impersonate="chrome131"
        ... )
    
    Notes:
        - http2 和 impersonate 使用不同的后端（httpx/curl_cffi），不能同时启用
        - 隐身模式的请求头优先级：用户自定义 > stealth 默认头
        - 使用 Workers 代理时，自定义请求头通过 X-CFSpider-Header-* 传递
    """
    # 应用随机延迟
    if delay:
        from .stealth import random_delay
        random_delay(delay[0], delay[1])
    
    params = kwargs.pop("params", None)
    headers = kwargs.pop("headers", {})
    
    # 如果启用隐身模式，添加完整的浏览器请求头
    if stealth:
        from .stealth import get_stealth_headers
        stealth_headers = get_stealth_headers(stealth_browser)
        # 用户自定义的 headers 优先级更高
        final_headers = stealth_headers.copy()
        final_headers.update(headers)
        headers = final_headers
    data = kwargs.pop("data", None)
    json_data = kwargs.pop("json", None)
    cookies = kwargs.pop("cookies", None)
    timeout = kwargs.pop("timeout", 30)
    
    # 记录请求开始时间
    start_time = time.time()
    
    # 如果指定了 impersonate，使用 curl_cffi
    if impersonate:
        response = _request_impersonate(
            method, url, cf_proxies, cf_workers, impersonate,
            params=params, headers=headers, data=data,
            json_data=json_data, cookies=cookies, timeout=timeout,
            **kwargs
        )
        _handle_map_output(response, url, start_time, map_output, map_file)
        return response
    
    # 如果启用 HTTP/2，使用 httpx
    if http2:
        response = _request_httpx(
            method, url, cf_proxies, cf_workers,
            params=params, headers=headers, data=data,
            json_data=json_data, cookies=cookies, timeout=timeout,
            **kwargs
        )
        _handle_map_output(response, url, start_time, map_output, map_file)
        return response
    
    # 如果没有指定 cf_proxies，直接使用 requests
    if not cf_proxies:
        resp = requests.request(
            method,
            url,
            params=params,
            headers=headers,
            data=data,
            json=json_data,
            cookies=cookies,
            timeout=timeout,
            **kwargs
        )
        response = CFSpiderResponse(resp)
        _handle_map_output(response, url, start_time, map_output, map_file)
        return response
    
    # cf_workers=False：使用普通代理
    if not cf_workers:
        # 处理代理格式
        proxy_url = cf_proxies
        if not proxy_url.startswith(('http://', 'https://', 'socks5://')):
            proxy_url = f"http://{proxy_url}"
        
        proxies = {
            "http": proxy_url,
            "https": proxy_url
        }
        
        resp = requests.request(
            method,
            url,
            params=params,
            headers=headers,
            data=data,
            json=json_data,
            cookies=cookies,
            timeout=timeout,
            proxies=proxies,
            **kwargs
        )
        response = CFSpiderResponse(resp)
        _handle_map_output(response, url, start_time, map_output, map_file)
        return response
    
    # cf_workers=True：使用 CFspider Workers API 代理
    cf_proxies_url = cf_proxies.rstrip("/")
    
    # 确保有协议前缀
    if not cf_proxies_url.startswith(('http://', 'https://')):
        cf_proxies_url = f"https://{cf_proxies_url}"
    
    target_url = url
    if params:
        target_url = f"{url}?{urlencode(params)}"
    
    proxy_url = f"{cf_proxies_url}/proxy?url={quote(target_url, safe='')}&method={method.upper()}"
    
    request_headers = {}
    if headers:
        for key, value in headers.items():
            request_headers[f"X-CFSpider-Header-{key}"] = value
    
    if cookies:
        cookie_str = "; ".join([f"{k}={v}" for k, v in cookies.items()])
        request_headers["X-CFSpider-Header-Cookie"] = cookie_str
    
    resp = requests.post(
        proxy_url,
        headers=request_headers,
        data=data,
        json=json_data,
        timeout=timeout,
        **kwargs
    )
    
    cf_colo = resp.headers.get("X-CF-Colo")
    cf_ray = resp.headers.get("CF-Ray")
    
    response = CFSpiderResponse(resp, cf_colo=cf_colo, cf_ray=cf_ray)
    _handle_map_output(response, url, start_time, map_output, map_file)
    return response


def _handle_map_output(response, url, start_time, map_output, map_file):
    """处理 IP 地图输出"""
    if not map_output:
        return
    
    # 计算响应时间
    response_time = (time.time() - start_time) * 1000  # 毫秒
    
    # 收集 IP 记录
    ip_map.add_ip_record(
        url=url,
        ip=None,  # 无法直接获取 IP，但有 cf_colo
        cf_colo=getattr(response, 'cf_colo', None),
        cf_ray=getattr(response, 'cf_ray', None),
        status_code=response.status_code,
        response_time=response_time
    )
    
    # 生成地图 HTML
    ip_map.generate_map_html(output_file=map_file)


def _request_impersonate(method, url, cf_proxies, cf_workers, impersonate,
                         params=None, headers=None, data=None, json_data=None,
                         cookies=None, timeout=30, **kwargs):
    """使用 curl_cffi 发送请求（支持 TLS 指纹模拟）"""
    curl_requests = _get_curl_cffi()
    
    # 如果没有指定 cf_proxies，直接请求
    if not cf_proxies:
        response = curl_requests.request(
            method,
            url,
            params=params,
            headers=headers,
            data=data,
            json=json_data,
            cookies=cookies,
            timeout=timeout,
            impersonate=impersonate,
            **kwargs
        )
        return CFSpiderResponse(response)
    
    # cf_workers=False：使用普通代理
    if not cf_workers:
        proxy_url = cf_proxies
        if not proxy_url.startswith(('http://', 'https://', 'socks5://')):
            proxy_url = f"http://{proxy_url}"
        
        response = curl_requests.request(
            method,
            url,
            params=params,
            headers=headers,
            data=data,
            json=json_data,
            cookies=cookies,
            timeout=timeout,
            impersonate=impersonate,
            proxies={"http": proxy_url, "https": proxy_url},
            **kwargs
        )
        return CFSpiderResponse(response)
    
    # cf_workers=True：使用 CFspider Workers API 代理
    cf_proxies = cf_proxies.rstrip("/")
    
    if not cf_proxies.startswith(('http://', 'https://')):
        cf_proxies = f"https://{cf_proxies}"
    
    target_url = url
    if params:
        target_url = f"{url}?{urlencode(params)}"
    
    proxy_url = f"{cf_proxies}/proxy?url={quote(target_url, safe='')}&method={method.upper()}"
    
    request_headers = {}
    if headers:
        for key, value in headers.items():
            request_headers[f"X-CFSpider-Header-{key}"] = value
    
    if cookies:
        cookie_str = "; ".join([f"{k}={v}" for k, v in cookies.items()])
        request_headers["X-CFSpider-Header-Cookie"] = cookie_str
    
    response = curl_requests.post(
        proxy_url,
        headers=request_headers,
        data=data,
        json=json_data,
        timeout=timeout,
        impersonate=impersonate,
        **kwargs
    )
    
    cf_colo = response.headers.get("X-CF-Colo")
    cf_ray = response.headers.get("CF-Ray")
    
    return CFSpiderResponse(response, cf_colo=cf_colo, cf_ray=cf_ray)


def _request_httpx(method, url, cf_proxies, cf_workers, params=None, headers=None,
                   data=None, json_data=None, cookies=None, timeout=30, **kwargs):
    """使用 httpx 发送请求（支持 HTTP/2）"""
    httpx = _get_httpx()
    
    # 如果没有指定 cf_proxies，直接请求
    if not cf_proxies:
        with httpx.Client(http2=True, timeout=timeout) as client:
            response = client.request(
                method,
                url,
                params=params,
                headers=headers,
                data=data,
                json=json_data,
                cookies=cookies,
                **kwargs
            )
            return CFSpiderResponse(response)
    
    # cf_workers=False：使用普通代理
    if not cf_workers:
        proxy_url = cf_proxies
        if not proxy_url.startswith(('http://', 'https://', 'socks5://')):
            proxy_url = f"http://{proxy_url}"
        
        with httpx.Client(http2=True, timeout=timeout, proxy=proxy_url) as client:
            response = client.request(
                method,
                url,
                params=params,
                headers=headers,
                data=data,
                json=json_data,
                cookies=cookies,
                **kwargs
            )
            return CFSpiderResponse(response)
    
    # cf_workers=True：使用 CFspider Workers API 代理
    cf_proxies = cf_proxies.rstrip("/")
    
    if not cf_proxies.startswith(('http://', 'https://')):
        cf_proxies = f"https://{cf_proxies}"
    
    target_url = url
    if params:
        target_url = f"{url}?{urlencode(params)}"
    
    proxy_url = f"{cf_proxies}/proxy?url={quote(target_url, safe='')}&method={method.upper()}"
    
    request_headers = {}
    if headers:
        for key, value in headers.items():
            request_headers[f"X-CFSpider-Header-{key}"] = value
    
    if cookies:
        cookie_str = "; ".join([f"{k}={v}" for k, v in cookies.items()])
        request_headers["X-CFSpider-Header-Cookie"] = cookie_str
    
    with httpx.Client(http2=True, timeout=timeout) as client:
        response = client.post(
            proxy_url,
            headers=request_headers,
            data=data,
            json=json_data,
            **kwargs
        )
    
    cf_colo = response.headers.get("X-CF-Colo")
    cf_ray = response.headers.get("CF-Ray")
    
    return CFSpiderResponse(response, cf_colo=cf_colo, cf_ray=cf_ray)


def get(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
        map_output=False, map_file="cfspider_map.html",
        stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """
    发送 GET 请求
    
    Args:
        url: 目标 URL
        cf_proxies: 代理地址
        cf_workers: 是否使用 Workers API（默认 True）
        http2: 是否启用 HTTP/2
        impersonate: TLS 指纹（如 "chrome131", "safari18_0", "firefox133"）
        map_output: 是否生成 IP 地图 HTML 文件
        map_file: 地图输出文件名
        stealth: 是否启用隐身模式（自动添加完整浏览器请求头）
        stealth_browser: 隐身模式浏览器类型（chrome/firefox/safari/edge/chrome_mobile）
        delay: 请求前随机延迟范围，如 (1, 3)
    """
    return request("GET", url, cf_proxies=cf_proxies, cf_workers=cf_workers, 
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def post(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
         map_output=False, map_file="cfspider_map.html",
         stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """发送 POST 请求"""
    return request("POST", url, cf_proxies=cf_proxies, cf_workers=cf_workers,
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def put(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
        map_output=False, map_file="cfspider_map.html",
        stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """发送 PUT 请求"""
    return request("PUT", url, cf_proxies=cf_proxies, cf_workers=cf_workers,
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def delete(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
           map_output=False, map_file="cfspider_map.html",
           stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """发送 DELETE 请求"""
    return request("DELETE", url, cf_proxies=cf_proxies, cf_workers=cf_workers,
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def head(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
         map_output=False, map_file="cfspider_map.html",
         stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """发送 HEAD 请求"""
    return request("HEAD", url, cf_proxies=cf_proxies, cf_workers=cf_workers,
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def options(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
            map_output=False, map_file="cfspider_map.html",
            stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """发送 OPTIONS 请求"""
    return request("OPTIONS", url, cf_proxies=cf_proxies, cf_workers=cf_workers,
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def patch(url, cf_proxies=None, cf_workers=True, http2=False, impersonate=None,
          map_output=False, map_file="cfspider_map.html",
          stealth=False, stealth_browser='chrome', delay=None, **kwargs):
    """发送 PATCH 请求"""
    return request("PATCH", url, cf_proxies=cf_proxies, cf_workers=cf_workers,
                   http2=http2, impersonate=impersonate,
                   map_output=map_output, map_file=map_file,
                   stealth=stealth, stealth_browser=stealth_browser, delay=delay, **kwargs)


def clear_map_records():
    """清空 IP 地图记录"""
    ip_map.clear_records()


def get_map_collector():
    """获取 IP 地图收集器"""
    return ip_map.get_collector()



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="cfspider",
    version="1.7.3",
    author="violettools",
    author_email="violet@violetteam.cloud",
    description="Cloudflare Workers proxy IP pool client with stealth mode, anti-detection, async, HTTP/2, TLS fingerprint, browser, mirror and IP map",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/violettoolssite/CFspider",
    project_urls={
        "Bug Tracker": "https://github.com/violettoolssite/CFspider/issues",
        "Documentation": "https://spider.violetteam.cloud",
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=[
        "requests>=2.20.0",
        "playwright>=1.40.0",
        "httpx[http2]>=0.25.0",
        "curl_cffi>=0.5.0",
        "beautifulsoup4>=4.9.0",
    ],
    entry_points={
        "console_scripts": [
            "cfspider=cfspider.cli:main",
        ],
    },
    keywords="cloudflare workers proxy ip pool crawler spider browser playwright httpx http2 async curl_cffi tls fingerprint impersonate map visualization maplibre mirror offline stealth anti-detection anti-bot",
)



"""
CFspider 命令行工具
"""

import sys
import subprocess


def install_browser():
    """
    安装 Chromium 浏览器
    
    Example:
        >>> import cfspider
        >>> cfspider.install_browser()
    """
    try:
        # 使用 playwright 命令行安装
        result = subprocess.run(
            [sys.executable, '-m', 'playwright', 'install', 'chromium'],
            capture_output=False
        )
        return result.returncode == 0
    except Exception as e:
        print(f"安装失败: {e}")
        return False


def main():
    """命令行入口"""
    if len(sys.argv) < 2:
        print_help()
        return
    
    command = sys.argv[1].lower()
    
    if command == 'install':
        print("正在安装 Chromium 浏览器...")
        if install_browser():
            print("安装完成!")
        else:
            print("安装失败，请检查网络连接或手动安装")
            sys.exit(1)
    
    elif command == 'version':
        from . import __version__
        print(f"cfspider {__version__}")
    
    elif command == 'help' or command == '-h' or command == '--help':
        print_help()
    
    else:
        print(f"未知命令: {command}")
        print_help()
        sys.exit(1)


def print_help():
    """打印帮助信息"""
    print("""
CFspider - Cloudflare 代理 IP 池

用法:
    cfspider <command>

命令:
    install     安装 Chromium 浏览器（用于 Browser 功能）
    version     显示版本号
    help        显示帮助信息

示例:
    cfspider install    # 安装浏览器
    cfspider version    # 显示版本

更多信息请访问: https://github.com/violettoolssite/CFspider
""")


if __name__ == '__main__':
    main()




"""
CFspider 隐身模式模块

提供完整的反爬虫规避能力，解决以下常见问题：

1. 请求头不完整或不真实
   - 问题：缺少 User-Agent, Accept-Language, Sec-Fetch-* 等头
   - 解决：自动添加 15+ 个真实浏览器请求头

2. 缺乏会话一致性
   - 问题：频繁更换 IP、User-Agent，不处理 Cookie
   - 解决：StealthSession 固定 User-Agent，自动管理 Cookie

3. 行为模式单一
   - 问题：只访问特定 API，没有随机停留等行为
   - 解决：random_delay 随机延迟，auto_referer 自动添加来源

使用方式：

    方式一：单次请求启用隐身模式
    >>> response = cfspider.get(url, stealth=True, stealth_browser='chrome')

    方式二：使用 StealthSession 保持会话一致性
    >>> with cfspider.StealthSession(browser='chrome', delay=(1, 3)) as session:
    ...     response1 = session.get(url1)  # 自动添加请求头
    ...     response2 = session.get(url2)  # 自动添加 Referer = url1

支持的浏览器：
    - chrome: Chrome 131（推荐，15 个请求头）
    - firefox: Firefox 133（12 个请求头，含隐私保护头）
    - safari: Safari 18（5 个请求头，macOS 风格）
    - edge: Edge 131（14 个请求头）
    - chrome_mobile: Chrome Mobile（10 个请求头，Android）
"""

import random
import time
from typing import Optional, Dict, List, Tuple, Any
from urllib.parse import urlparse


# Chrome 131 完整请求头模板
CHROME_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
    'Accept-Encoding': 'gzip, deflate, br, zstd',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Sec-CH-UA': '"Google Chrome";v="131", "Chromium";v="131", "Not_A Brand";v="24"',
    'Sec-CH-UA-Mobile': '?0',
    'Sec-CH-UA-Platform': '"Windows"',
    'Upgrade-Insecure-Requests': '1',
    'Cache-Control': 'max-age=0',
    'Connection': 'keep-alive',
    'DNT': '1',
}

# Firefox 133 完整请求头模板
FIREFOX_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:133.0) Gecko/20100101 Firefox/133.0',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/png,image/svg+xml,*/*;q=0.8',
    'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
    'Accept-Encoding': 'gzip, deflate, br, zstd',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Upgrade-Insecure-Requests': '1',
    'Connection': 'keep-alive',
    'DNT': '1',
    'Sec-GPC': '1',
}

# Safari 18 完整请求头模板
SAFARI_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/18.0 Safari/605.1.15',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Language': 'zh-CN,zh-Hans;q=0.9',
    'Accept-Encoding': 'gzip, deflate, br',
    'Connection': 'keep-alive',
}

# Edge 131 完整请求头模板
EDGE_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36 Edg/131.0.0.0',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
    'Accept-Encoding': 'gzip, deflate, br, zstd',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Sec-CH-UA': '"Microsoft Edge";v="131", "Chromium";v="131", "Not_A Brand";v="24"',
    'Sec-CH-UA-Mobile': '?0',
    'Sec-CH-UA-Platform': '"Windows"',
    'Upgrade-Insecure-Requests': '1',
    'Cache-Control': 'max-age=0',
    'Connection': 'keep-alive',
}

# 移动端 Chrome 请求头
CHROME_MOBILE_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Linux; Android 14; Pixel 8 Pro) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Mobile Safari/537.36',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8',
    'Accept-Language': 'zh-CN,zh;q=0.9',
    'Accept-Encoding': 'gzip, deflate, br',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Sec-CH-UA': '"Google Chrome";v="131", "Chromium";v="131"',
    'Sec-CH-UA-Mobile': '?1',
    'Sec-CH-UA-Platform': '"Android"',
    'Upgrade-Insecure-Requests': '1',
}

# 浏览器配置集合
BROWSER_PROFILES = {
    'chrome': CHROME_HEADERS,
    'firefox': FIREFOX_HEADERS,
    'safari': SAFARI_HEADERS,
    'edge': EDGE_HEADERS,
    'chrome_mobile': CHROME_MOBILE_HEADERS,
}

# 默认使用 Chrome
DEFAULT_BROWSER = 'chrome'


def get_stealth_headers(browser: str = 'chrome', custom_headers: Dict = None) -> Dict[str, str]:
    """
    获取隐身模式请求头
    
    Args:
        browser: 浏览器类型 (chrome/firefox/safari/edge/chrome_mobile)
        custom_headers: 自定义请求头（会覆盖默认值）
    
    Returns:
        完整的浏览器请求头字典
    """
    headers = BROWSER_PROFILES.get(browser, CHROME_HEADERS).copy()
    if custom_headers:
        headers.update(custom_headers)
    return headers


def get_random_browser_headers() -> Dict[str, str]:
    """随机选择一个浏览器的请求头"""
    browser = random.choice(list(BROWSER_PROFILES.keys()))
    return get_stealth_headers(browser)


def random_delay(min_sec: float = 0.5, max_sec: float = 2.0) -> float:
    """
    随机延迟，模拟人类行为
    
    Args:
        min_sec: 最小延迟秒数
        max_sec: 最大延迟秒数
    
    Returns:
        实际延迟的秒数
    """
    delay = random.uniform(min_sec, max_sec)
    time.sleep(delay)
    return delay


def get_referer(current_url: str, previous_url: str = None) -> Optional[str]:
    """
    生成 Referer 头
    
    Args:
        current_url: 当前请求的 URL
        previous_url: 上一个访问的 URL
    
    Returns:
        Referer 值
    """
    if previous_url:
        return previous_url
    
    # 如果没有上一个 URL，使用当前 URL 的首页作为 Referer
    parsed = urlparse(current_url)
    return f"{parsed.scheme}://{parsed.netloc}/"


def update_sec_fetch_headers(headers: Dict, site_type: str = 'none') -> Dict:
    """
    更新 Sec-Fetch-* 请求头
    
    Args:
        headers: 原始请求头
        site_type: 网站类型 (none/same-origin/same-site/cross-site)
    
    Returns:
        更新后的请求头
    """
    headers = headers.copy()
    headers['Sec-Fetch-Site'] = site_type
    
    if site_type == 'none':
        # 直接访问（如在地址栏输入URL）
        headers['Sec-Fetch-Mode'] = 'navigate'
        headers['Sec-Fetch-Dest'] = 'document'
    elif site_type in ('same-origin', 'same-site'):
        # 站内跳转
        headers['Sec-Fetch-Mode'] = 'navigate'
        headers['Sec-Fetch-Dest'] = 'document'
    else:
        # 跨站跳转
        headers['Sec-Fetch-Mode'] = 'navigate'
        headers['Sec-Fetch-Dest'] = 'document'
    
    return headers


class StealthSession:
    """
    隐身会话类
    
    提供完整的会话一致性管理，解决反爬虫检测的三大问题：
    
    1. 固定 User-Agent：整个会话使用同一个浏览器指纹
    2. 自动管理 Cookie：响应中的 Cookie 自动保存并在后续请求中发送
    3. 自动添加 Referer：页面跳转时自动添加来源信息
    4. 随机延迟：每次请求前随机等待，模拟人类行为
    5. 自动更新 Sec-Fetch-Site：根据 Referer 判断同站/跨站访问
    
    Attributes:
        browser (str): 当前使用的浏览器类型
        cf_proxies (str): 代理地址
        delay (tuple): 随机延迟范围
        auto_referer (bool): 是否自动添加 Referer
        last_url (str): 上一次请求的 URL
        request_count (int): 会话累计请求次数
    
    Example:
        >>> import cfspider
        >>> 
        >>> # 基本用法
        >>> with cfspider.StealthSession(browser='chrome') as session:
        ...     # 第一次请求：Sec-Fetch-Site: none
        ...     r1 = session.get("https://example.com")
        ...     
        ...     # 第二次请求：自动添加 Referer: https://example.com
        ...     # Sec-Fetch-Site: same-origin
        ...     r2 = session.get("https://example.com/page2")
        >>> 
        >>> # 带随机延迟
        >>> with cfspider.StealthSession(delay=(1, 3)) as session:
        ...     for url in urls:
        ...         # 每次请求前随机等待 1-3 秒
        ...         response = session.get(url)
        >>> 
        >>> # 结合代理使用
        >>> with cfspider.StealthSession(
        ...     cf_proxies="https://your-workers.dev",
        ...     browser='firefox',
        ...     delay=(0.5, 2.0)
        ... ) as session:
        ...     response = session.get("https://example.com")
        ...     print(f"请求次数: {session.request_count}")
        ...     print(f"当前 Cookie: {session.get_cookies()}")
    
    Note:
        StealthSession 与普通 Session 的区别：
        - Session: 仅保持代理配置和基本请求头
        - StealthSession: 完整的隐身模式，包括浏览器指纹、Cookie 管理、
                          自动 Referer、随机延迟、Sec-Fetch-* 更新
    """
    
    def __init__(
        self,
        browser: str = 'chrome',
        cf_proxies: str = None,
        cf_workers: bool = True,
        delay: Tuple[float, float] = None,
        auto_referer: bool = True,
        **kwargs
    ):
        """
        初始化隐身会话
        
        Args:
            browser (str): 浏览器类型，决定使用的 User-Agent 和请求头模板
                - 'chrome': Chrome 131（推荐，最完整的请求头，15 个）
                - 'firefox': Firefox 133（含 Sec-GPC 隐私头，12 个）
                - 'safari': Safari 18（macOS 风格，5 个）
                - 'edge': Edge 131（类似 Chrome，14 个）
                - 'chrome_mobile': Chrome Mobile（Android，10 个）
            cf_proxies (str, optional): 代理地址
                - 不指定则直接请求目标 URL
                - 指定 Workers 地址时配合 cf_workers=True
                - 指定普通代理时配合 cf_workers=False
            cf_workers (bool): 是否使用 CFspider Workers API（默认 True）
            delay (tuple, optional): 请求间随机延迟范围（秒）
                - 如 (1, 3) 表示每次请求前随机等待 1-3 秒
                - 第一次请求不会延迟
                - 用于避免请求频率过高被检测
            auto_referer (bool): 是否自动添加 Referer（默认 True）
                - True: 自动使用上一个 URL 作为 Referer
                - False: 不自动添加（但可以手动指定）
            **kwargs: 保留参数，用于未来扩展
        
        Example:
            >>> session = cfspider.StealthSession(
            ...     browser='chrome',
            ...     cf_proxies='https://your-workers.dev',
            ...     delay=(1, 3),
            ...     auto_referer=True
            ... )
        """
        self.browser = browser
        self.cf_proxies = cf_proxies
        self.cf_workers = cf_workers
        self.delay = delay
        self.auto_referer = auto_referer
        self.last_url = None
        self.request_count = 0
        self._extra_kwargs = kwargs
        
        # 获取固定的浏览器请求头
        self._base_headers = get_stealth_headers(browser)
        
        # Cookie 管理
        self._cookies = {}
    
    def _prepare_headers(self, url: str, headers: Dict = None) -> Dict:
        """准备请求头"""
        final_headers = self._base_headers.copy()
        
        # 添加 Referer
        if self.auto_referer and self.last_url:
            parsed_current = urlparse(url)
            parsed_last = urlparse(self.last_url)
            
            if parsed_current.netloc == parsed_last.netloc:
                # 同站跳转
                final_headers['Referer'] = self.last_url
                final_headers = update_sec_fetch_headers(final_headers, 'same-origin')
            else:
                # 跨站跳转
                final_headers['Referer'] = self.last_url
                final_headers = update_sec_fetch_headers(final_headers, 'cross-site')
        
        # 合并自定义请求头
        if headers:
            final_headers.update(headers)
        
        return final_headers
    
    def _apply_delay(self):
        """应用请求延迟"""
        if self.delay and self.request_count > 0:
            random_delay(self.delay[0], self.delay[1])
    
    def _update_cookies(self, response):
        """更新 Cookie"""
        if hasattr(response, 'cookies'):
            for cookie in response.cookies:
                self._cookies[cookie.name] = cookie.value
    
    def get(self, url: str, **kwargs) -> Any:
        """
        发送 GET 请求
        
        Args:
            url: 目标 URL
            **kwargs: 其他参数
        
        Returns:
            响应对象
        """
        from .api import get as _get
        
        self._apply_delay()
        
        headers = self._prepare_headers(url, kwargs.pop('headers', None))
        
        # 添加 Cookie
        cookies = kwargs.pop('cookies', {})
        cookies.update(self._cookies)
        
        response = _get(
            url,
            cf_proxies=self.cf_proxies,
            cf_workers=self.cf_workers,
            headers=headers,
            cookies=cookies,
            **kwargs
        )
        
        self._update_cookies(response)
        self.last_url = url
        self.request_count += 1
        
        return response
    
    def post(self, url: str, **kwargs) -> Any:
        """发送 POST 请求"""
        from .api import post as _post
        
        self._apply_delay()
        
        headers = self._prepare_headers(url, kwargs.pop('headers', None))
        
        # POST 请求的特殊头
        if 'json' in kwargs or 'data' in kwargs:
            headers.setdefault('Content-Type', 'application/x-www-form-urlencoded')
        
        cookies = kwargs.pop('cookies', {})
        cookies.update(self._cookies)
        
        response = _post(
            url,
            cf_proxies=self.cf_proxies,
            cf_workers=self.cf_workers,
            headers=headers,
            cookies=cookies,
            **kwargs
        )
        
        self._update_cookies(response)
        self.last_url = url
        self.request_count += 1
        
        return response
    
    def put(self, url: str, **kwargs) -> Any:
        """发送 PUT 请求"""
        from .api import put as _put
        
        self._apply_delay()
        headers = self._prepare_headers(url, kwargs.pop('headers', None))
        cookies = kwargs.pop('cookies', {})
        cookies.update(self._cookies)
        response = _put(
            url,
            cf_proxies=self.cf_proxies,
            cf_workers=self.cf_workers,
            headers=headers,
            cookies=cookies,
            **kwargs
        )
        self._update_cookies(response)
        self.last_url = url
        self.request_count += 1
        return response
    
    def delete(self, url: str, **kwargs) -> Any:
        """发送 DELETE 请求"""
        from .api import delete as _delete
        
        self._apply_delay()
        headers = self._prepare_headers(url, kwargs.pop('headers', None))
        cookies = kwargs.pop('cookies', {})
        cookies.update(self._cookies)
        response = _delete(
            url,
            cf_proxies=self.cf_proxies,
            cf_workers=self.cf_workers,
            headers=headers,
            cookies=cookies,
            **kwargs
        )
        self._update_cookies(response)
        self.last_url = url
        self.request_count += 1
        return response
    
    def head(self, url: str, **kwargs) -> Any:
        """发送 HEAD 请求"""
        from .api import head as _head
        
        self._apply_delay()
        headers = self._prepare_headers(url, kwargs.pop('headers', None))
        cookies = kwargs.pop('cookies', {})
        cookies.update(self._cookies)
        response = _head(
            url,
            cf_proxies=self.cf_proxies,
            cf_workers=self.cf_workers,
            headers=headers,
            cookies=cookies,
            **kwargs
        )
        self._update_cookies(response)
        self.last_url = url
        self.request_count += 1
        return response
    
    def get_cookies(self) -> Dict[str, str]:
        """获取当前会话的所有 Cookie"""
        return self._cookies.copy()
    
    def set_cookie(self, name: str, value: str):
        """设置 Cookie"""
        self._cookies[name] = value
    
    def clear_cookies(self):
        """清除所有 Cookie"""
        self._cookies.clear()
    
    def get_headers(self) -> Dict[str, str]:
        """获取当前会话的基础请求头"""
        return self._base_headers.copy()
    
    def close(self):
        """关闭会话"""
        pass  # 无需清理，每次请求都是独立的
    
    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


# 支持的浏览器列表
SUPPORTED_BROWSERS = list(BROWSER_PROFILES.keys())


def get_supported_browsers() -> List[str]:
    """获取支持的浏览器列表"""
    return SUPPORTED_BROWSERS.copy()

